/*	file name *** sys_sim.c***     */
/*  Simulation of 4th order system     */
/*  *** Inverted pendulum ***          */

/*  The state equations of the system must be placed */
/*  in filt() function!!!                            */

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>

/*
/ 	Main definitions
*/
#define ARGS	  2
/* If needed adjust the following constsnts properly */
/**********************************************************/
#define RSIZE     5     // Number of state equations+1
#define END_TIME 2.0   // Simulation end time
#define T_STEP 0.01     // Time step
/**********************************************************/
/* 
/     Car and pendulum parameters (system matrix)
*/
#define M_P   0.2      // Pendulum weigth [kg]
#define M_C   1.0      // Car weigth ]kg]
#define D     0.003    // Car friction coeff.
#define P_LEN 0.8      // Pendulum lenght [m]
#define J_P   1.0/3.0*M_P*P_LEN*P_LEN // Pendulum moment of inertia [kg*m^2]
#define G     9.81     // Gravitational acceleration [kg*m/s^2]
#define A11   0.0
#define A12   0.0
#define A13   1.0
#define A14   0.0
#define A21   0.0
#define A22   0.0
#define A23   0.0
#define A24   1.0
#define A31   0.0
#define A32   -3.0*M_P*G/(4.0*M_C+M_P)
#define A33   -4.0*D/(4.0*M_C+M_P)
#define A34   0.0
#define A41   0.0
#define A42   3.0*(M_C+M_P)*G/((4.0*M_C+M_P)*P_LEN)
#define A43   3.0*D/((4.0*M_C+M_P)*P_LEN)
#define A44   0.0
#define B1    0.0
#define B2    0.0
#define B3    4/(4.0*M_C+M_P)
#define B4    -3.0/((4.0*M_C+M_P)*P_LEN)

/*
/	Global declations
*/
double rtime, out, input;
double lhv[RSIZE], rhv[RSIZE];

void main (int argc, char *argv[])
{
    FILE *fp, *fopen();
    extern void error(char *, char *);
    extern void runge_cutta(void);
    int i;
    if(argc < ARGS)
      error("Usage: output_file", NULL);
    if((fp = fopen(argv[1], "w")) == NULL)
      error("est: can't open %s", argv[2]);
    while (rtime <= END_TIME){
	runge_cutta();
	fprintf(fp, "%f %f %f %f %f %f\n", rtime, input, rhv[1], rhv[2], rhv[3], rhv[4]);
	rtime += T_STEP;
    }
}

void runge_cutta(void)
{
extern void filt(void);
    int i;
    double a, b;			 /* coeficients */
    double fn, r, h;		 /* program's variables */
    double work[RSIZE];
    h = T_STEP; 	 /* step */
    a = 1.0 - sqrt((double)0.5);
    b = 2.0 - a;
    for (i = 0;i < RSIZE;i++){
	work[i] = 0.0;           /* Clear work area */
    }
    rhv[0] = rtime;
    lhv[0] = 1.0;
    filt();
    for (i = 0;i < RSIZE;i++){
	fn = h * lhv[i];
	r = 0.5 * fn - work[i];
	work[i] = work[i] + 3.0 * r - 0.5 * fn;
	rhv[i] += r;
    }
    filt();
    for (i = 0;i < RSIZE;i++){
	fn = h * lhv[i];
	r = a * (fn - work[i]);
	work[i] = work[i] + 3.0 * r - a * fn;
	rhv[i] += r;
    }
    filt();
    for (i = 0;i < RSIZE;i++){
	fn = h * lhv[i];
	r = b * (fn - work[i]);
	work[i] = work[i] + 3.0 * r - b * fn;
	rhv[i] += r;
    }
    filt();
    for (i = 0;i < RSIZE;i++){
	fn = h * lhv[i];
	r = (fn - 2.0 * work[i]) / 6.0;
	rhv[i] += r;
    }
}

void filt(void)
{
  /* Car and pendulum state equations */
    	lhv[1] = A13 * rhv[3];
    	lhv[2] = A24 * rhv[4];
	lhv[3] = A32 * rhv[2] + A33 * rhv[3] + B3 * input;
	lhv[4] = A42 * rhv[2] + A43 * rhv[3] + B4 * input;
	out = rhv[1];
}

void error(s0, s1)
     char *s0, *s1;
{
    printf(s0,s1);
    printf("\n");
    exit(1);
}

